@@ -1,7 +1,7 @@
 import functools
 from contextlib import ExitStack
 from types import FunctionType, GeneratorType
-from typing import List, Dict, Union, Tuple, Callable
+from typing import List, Dict, Union, Tuple, Callable, Any
 
 from testbook import testbook
 
@@ -15,25 +15,42 @@ TRACK_TYPE = Dict[str,
 EXECUTE_TYPE = Union[Tuple[str],
                      Tuple[str, str],
                      str,
-                     FunctionType,
+                     Callable[[JPTestBook], Any],
                      List['EXECUTE_TYPE'],
                      Dict[str, Union['EXECUTE_TYPE', TRACK_TYPE]]]
 
 
 class JPTest(testbook):
+    """
+    decorator to use with test functions
+    """
     NOTEBOOK: str = None
     TESTS: List[Tuple['JPTest', Callable, Callable]] = []
 
-    def __init__(self, name: str = None, max_score: Union[float, int] = None, execute: EXECUTE_TYPE = None, **kwargs):
-        super().__init__(self.NOTEBOOK, execute)
+    def __init__(self, name: str = None, max_score: Union[float, int] = None,
+                 execute: EXECUTE_TYPE = None, timeout: int = 120,
+                 **kwargs):
+        """
+        :param name: name used in the output
+        :param max_score: maximum score (can be exceeded, used to calculate total score)
+        :param execute: cells or code to execute prior to the test
+        :param timeout: execution timeout in seconds (default: 2 minutes)
+        :param kwargs: any named parameters are copied to the JPTestBook object
+        """
+        super().__init__(self.NOTEBOOK, execute, timeout=timeout)
 
         self.tb: JPTestBook = JPTestBook(self.client, **kwargs)
         self.name: str = name
         self.max_score: float = None if max_score is None else float(max_score)
 
     def _prepare_recursive(self, item: EXECUTE_TYPE) -> List[JPTestParams]:
+        """
+        :param item: item to execute
+        :return: a list of JPTestParams if any track directives were used
+        """
         extracted: List[JPTestParams] = []
 
+        # dictionaries
         if isinstance(item, dict):
             with ExitStack() as stack:
                 # create function wrappers
@@ -51,20 +68,31 @@ class JPTest(testbook):
                 for ep in self._prepare_recursive(item['execute']):
                     extracted.append(ep)
 
+        # tuples
         elif isinstance(item, tuple):
+            # two items: execute every cell from the first occurrence of the first tag
+            # to the first occurrence of the second tag
             if len(item) == 2:
                 self.client.execute_cell(slice(item[0], item[1]))
+
+            # one item: execute every cell with this tag
             elif len(item) == 1:
-                self.client.execute_cell(item[0])
+                for idx, cell in enumerate(self.client.cells):
+                    if 'tags' in cell['metadata'] and item[0] in cell['metadata']['tags']:
+                        self.client.execute_cell(idx)
+
             else:
                 raise ValueError('unsupported tuple length')
 
+        # string
         elif isinstance(item, str):
             self.client.inject(item)
 
+        # function
         elif isinstance(item, FunctionType):
-            item(self)
+            item(self.tb)
 
+        # list
         elif isinstance(item, list):
             for i in item:
                 for ep in self._prepare_recursive(i):
